#!/usr/bin/perl -w

package NCS::Alert;
use strict;

use NCS::AlertTarget;

sub new {
	my $self = shift;
	my $sys = shift;
	my $alert_id = shift;
	my $name = shift;
	my $enabled = shift;

	my $alert = {
		sys		=> $sys,
		alert_id	=> $alert_id,
		name		=> $name,
		enabled		=> $enabled,
		targets		=> undef
	};

	bless($alert, $self);
	return $alert;
}

sub load_targets {
	my $self = shift;

	$self->{sys}->{logmgr}->do_debug("Alert: $self->{name}: reading targets...");

	my $h = $self->{sys}->{db}->execute_query("SELECT `target_id` FROM `targets` WHERE `alert_id` = ?", $self->{alert_id});
	return undef if (!$h);

	# Parse the targets...
	#
	my @targets = ();
	while (defined(my $row = $h->fetchrow_arrayref))
	{
		my $target = new NCS::AlertTarget(\%{ $self->{sys} }, $$row[0], $self->{name});
		push(@targets, $target);
	}

	$self->{targets} = \@targets;
	$self->{sys}->{logmgr}->do_debug("Alert: $self->{name}: load_targets finished");

	return 1;
}

sub process_targets {
	my $self = shift;

	$self->{sys}->{logmgr}->do_debug("Alert: $self->{name}: process_targets() called, alert_id=$self->{alert_id}, ena=$self->{enabled}");
	return unless ($self->{enabled} == 1);

	#
	# Let's see what devices belong to this alert group first
	# state_code 3 = initial, state_code 4 = changed
	#
	my $h = $self->{sys}->{db}->execute_query("SELECT d.device_id,d.revision_id,d.hostname,d.ipaddress,".
		"(SELECT r.revision FROM revisions AS r WHERE r.revision_id = d.revision_id) AS cur_rev,".
		"(SELECT r.revision_id FROM revisions AS r WHERE r.device_id = d.device_id AND r.revision_id != d.revision_id ORDER BY r.revision DESC LIMIT 0,1) AS prev_id,".
		"(SELECT r.revision FROM revisions AS r WHERE r.revision_id = prev_id) AS prev_rev ".
		"FROM devices AS d WHERE d.alert_id = ? AND (d.state_code = 3 OR d.state_code = 4);", $self->{alert_id});
	if (!$h)
	{
		$self->{sys}->{logmgr}->do_debug("Alert: $self->{name}: change recordset invalid, aborting");
		return;
	}
	$self->{sys}->{logmgr}->do_debug("Alert: $self->{name}: process_target() change rows=".$h->rows);

	my @change = ();
	while (defined(my $row = $h->fetchrow_hashref))
	{
		$self->{sys}->{logmgr}->do_debug("Alert: $self->{name}: got change, hostname=$$row{hostname}");
		push(@change, $row);
	}

	#
	# Now lets find any errors for this alert group.
	# state_code 6 = error, state_code 7 = syserr
	#

	$h = $self->{sys}->{db}->execute_query("SELECT `device_id`,`hostname`,`ipaddress`,`state_msg` ".
			"FROM `devices` WHERE `alert_id` = ? AND `state_code` = 6 ORDER BY `hostname` ASC", $self->{alert_id});
	if (!$h)
	{
		$self->{sys}->{logmgr}->do_debug("Alert: $self->{name}: error recordset invalid, aborting");
		return;
	}
	$self->{sys}->{logmgr}->do_debug("Alert: $self->{name}: process_target() error rows=".$h->rows);

	my @error = ();
	while (defined(my $row = $h->fetchrow_hashref))
	{
		$self->{sys}->{logmgr}->do_debug("Alert: $self->{name}: got error, hostname=$$row{hostname}");
		push(@error, $row);
	}

	#
	# Finally, walk our target list and pass each target the change/error arrays.
	#

	my $alertCount = scalar(@change) + scalar(@error);

	if ($alertCount < 1)
	{
		$self->{sys}->{logmgr}->do_debug("Alert: $self->{name}: process_targets() alertCount=$alertCount, finishing early.");
		return;
	}

	$self->{sys}->{logmgr}->do_debug("Alert: $self->{name}: process_targets() alertCount=$alertCount, processing targets...");
	foreach my $t (@{$self->{targets}})
	{
		$self->{sys}->{logmgr}->do_debug("Alert: $self->{name}: process_targets() processing target_id=$t->{target_id}");
		$t->process_alert(\@change, \@error);
	}

}

1;