#!/usr/bin/perl -w

package NCS::AlertTarget;
use strict;

use MIME::Lite;

my %alertFlags = (
	changed	=> 0x0001,
	error	=> 0x0002
);

my @alertHeader = (
	'<html><head></head><body style="background-color: #fff; margin: 0; padding: 15px; font-family: Verdana, sans-serif; font-size: 8pt;">',
	'<h1 style="font-size: 12pt; color: #203E6B; border-bottom: 1px solid #203E6B;">Automated Status Alert</h1>'
);
my @alertFooter = (
	'<p style="color: #999; border-top: 1px solid #999;">Copyright &copy; 2006-2008 Alasdair McWilliam.</p>',
	'</body></html>'
);

my $alertTableHeader = '<table width="100%" cellpadding="2" style="border-collapse: collapse; border: 1px solid #203E6B;">';
my $alertTableFooter = '</table>';

my @alertChangeHRow = (
	'<tr style="background-color: #D8E4FF; font-weight: bold; font-size: 8pt;">',
	'<td>Hostname</td><td>IP Address</td><td>New Revision</td><td>Compare to Previous</td>',
	'</tr>'
);
my @alertErrorHRow = (
	'<tr style="background-color: #D8E4FF; font-weight: bold; font-size: 8pt;">',
	'<td>Hostname</td><td>IP Address</td><td>Details</td>',
	'</tr>'
);

my $alertInitialRow = '<tr style="font-size: 8pt; padding: 2px;"><td>%s</td><td>%s</td><td>%s</td><td>Initial Configuration</td></tr>';
my $alertChangeRow = '<tr style="font-size: 8pt; padding: 2px;"><td>%s</td><td>%s</td><td>%s</td><td><a href="%s/Devices/Compare.php?from=%s&amp;to=%s&amp;diffwin">Compare to r%s</a></td></tr>';
my $alertErrorRow = '<tr style="font-size: 8pt; padding: 2px;"><td>%s</td><td>%s</td><td>%s</td></tr>';

sub new {
	my $self = shift;
	my $sys = shift;
	my $target_id = shift;
	my $alert_name = shift; # Kludge

	my $rec = $sys->{db}->get_record("SELECT * FROM `targets` WHERE `target_id` = ?", $target_id);
	return undef if (!$rec);

	my $target = {
		sys		=> $sys,
		target_id	=> $target_id,
		alert_name	=> $alert_name,
		alert_id	=> $$rec{alert_id},
		email_address	=> $$rec{email_address},
		flags		=> $$rec{flags}
	};
	bless($target, $self);

	$sys->{logmgr}->do_debug("AlertTarget: t_id=$target_id a_id=$target->{alert_id} e_addr=$target->{email_address} flags=$target->{flags}");
	return $target;
}

sub process_alert {
	my $self = shift;
	my $changes = shift;
	my $errors = shift;

	my @changeHtml = ();
	my $changeCount = 0;
	my @errorHtml = ();
	my $errorCount = 0;

	$self->{sys}->{logmgr}->do_debug("AlertTarget: $self->{target_id}: process_alert() email_addr=$self->{email_address} flags=$self->{flags}");

	if ($self->{flags} & $alertFlags{changed})
	{
		$self->{sys}->{logmgr}->do_debug("AlertTarget: $self->{target_id}: process_alert() [CHANGE FLAG] processing changes...");

		push(@changeHtml, '<h2 style="font-size: 10pt;">Configuration Changes</h2>');
		push(@changeHtml, $alertTableHeader, @alertChangeHRow);

		foreach my $chg (@{$changes})
		{
			push(@changeHtml, sprintf(($$chg{cur_rev} == 1 ? $alertInitialRow : $alertChangeRow),
					$$chg{hostname}, $$chg{ipaddress}, $$chg{cur_rev}, $self->{sys}->{loader}->{url},
					$$chg{prev_id}, $$chg{revision_id}, $$chg{prev_rev}));
			$changeCount++;
		}

		push(@changeHtml, $alertTableFooter);
	}

	if ($self->{flags} & $alertFlags{error})
	{
		$self->{sys}->{logmgr}->do_debug("AlertTarget: $self->{target_id}: process_alert() [ERROR FLAG] processing changes...");

		push(@errorHtml, '<h2 style="font-size: 10pt;">Transfer Errors</h2>');
		push(@errorHtml, $alertTableHeader, @alertErrorHRow);

		foreach my $err (@{$errors})
		{
			push(@errorHtml, sprintf($alertErrorRow, $$err{hostname}, $$err{ipaddress}, $$err{state_msg}));
			$errorCount++;
		}

		push(@errorHtml, $alertTableFooter);
	}

	#
	# We do check we have at least one alert to send before the target list of an alert
	# group is processed. However that check can't tell what flags a target has set. This means
	# we need to check what we're going to send again here, and bail if necessary.
	#

	if ($changeCount == 0 && $errorCount == 0)
	{
		$self->{sys}->{logmgr}->do_debug("AlertTarget: $self->{target_id}: process_alert() finished processing, no alerts for target, finish early.");
		return;
	}

	#
	# We now have to arrays containg a HTML table... build an e-mail.
	#

	my $html = join("\r\n", @alertHeader, "<p>The following devices changed state at <strong>".$self->{sys}->{util}->nowstr."</strong>.</p>");

	if ($changeCount > 0) {
		$html .= join("\r\n", @changeHtml);
	}
	if ($errorCount > 0) {
		$html .= join("\r\n", @errorHtml);
	}

	$html .= join("\r\n", @alertFooter);

	#
	# Now build a MIME object and mail this out to the target!s
	#

	my $m = new MIME::Lite(
				From => $self->{sys}->{loader}->{alert_source},
				To => $self->{email_address},
				Subject => "Automated Status Alert [$self->{alert_name}]",
				Type => "multipart/related"
			);
	if (!$m)
	{
		$self->{sys}->{logmgr}->do_debug("AlertTarget: mime object invalid, aborting");
	}
	else
	{
		$self->{sys}->{logmgr}->do_debug("AlertTarget: $self->{target_id}: process_alert() MIME ready, change=$changeCount error=$errorCount, sending...");
		$m->attach(Type => "text/html", Data => $html);
		$m->send("smtp", $self->{sys}->{loader}->{alert_smtp_server}, Timeout => 30);
	}
}

1;
