#!/usr/bin/perl -w
#
# Controller module for Nortel Alteon Web Switches - eg Alteon AD3.
#
# Supported from NCS Version 2.2.0.
#
# OID Sources:
#     http://source.altinity.org/source/mibs/ALTEON-TIGON-SWITCH-MIB.mib
#     http://www.oidview.com/mibs/1872/ALTEON-TIGON-SWITCH-MIB.html
#

package NCS::AlteonMIB;
use strict;

use Net::SNMP;

my %Alteon = (
	action		=> ".1.3.6.1.4.1.1872.2.1.2.1.21.0",
	filename	=> ".1.3.6.1.4.1.1872.2.1.2.1.19.0",
	tftpserver	=> ".1.3.6.1.4.1.1872.2.1.2.1.18.0",
	tftpstatus	=> ".1.3.6.1.4.1.1872.2.1.2.1.22.0",
);

my %AlteonAction = (
	other		=> "1",
	imgget		=> "2",
	cfgget		=> "3",
	cfgput		=> "4", # We put config onto the TFTP Server
	dumpput		=> "5",
);

sub new
{
	my $self = shift;
	my $sys = shift;
	my $device = shift;

	my $mib = {
		sys	=> $sys,
		device	=> $device,

		dstip	=> undef,
		dstfile	=> undef,

		error	=> "",
	};
	bless($mib, $self);

	$sys->{logmgr}->do_debug("AlteonMIB: $device->{hostname}: successfully initialised");
	return $mib;
}

sub set_destination {
	my $self = shift;

	$self->{dstip} = shift;
	$self->{dstfile} = shift;

	$self->{sys}->{logmgr}->do_debug("AlteonMIB: $self->{device}->{hostname}: set_destination() dstip=$self->{dstip}, dstfile=$self->{dstfile}");
}

sub retrieve_config {
	my $self = shift;

	$self->{error} = undef;

	$self->{sys}->{logmgr}->do_debug("AlteonMIB: $self->{device}->{hostname}: doing SNMP Set, agTftpAction=cfgput");
	$self->{device}->{snmp}->set(
			"$Alteon{filename}",	OCTET_STRING,	$self->{dstfile},
			"$Alteon{tftpserver}",	OCTET_STRING,	$self->{dstip},
			"$Alteon{action}",	INTEGER,	$AlteonAction{cfgput}
		);

	if (defined($self->{device}->{snmp}->{error}) && $self->{device}->{snmp}->{error} ne "") {
		$self->{error} = "SNMP Set Failed: $self->{device}->{snmp}->{error}";
	}
	else {
		my $final_result = undef;

		$self->{sys}->{logmgr}->do_debug("AlteonMIB: $self->{device}->{hostname}: set OK, retrieve_config() reading agTftpLastActionStatus");
		$final_result = $self->{device}->{snmp}->get("$Alteon{tftpstatus}");
		$self->{sys}->{logmgr}->do_debug("AlteonMIB: $self->{device}->{hostname}: agTftpLastActionStatus=".((defined($final_result)) ? $final_result : "NULL!"));

		if (!defined($final_result)) {
			$final_result = "Unknown copy fail code";
		}

		if ($final_result !~ /Success/) {
			$self->{error} = "TFTP Transfer Failed: $final_result";
		}

		$self->{sys}->{logmgr}->do_debug("AlteonMIB: $self->{device}->{hostname}: retrieve_config() finished, final_result=$final_result");
	}

	$self->{sys}->{logmgr}->do_debug("AlteonMIB: $self->{device}->{hostname}: retrieve_config() finished, error=".defined($self->{error}) ? $self->{error} : "NULL ERROR");
}

sub filter_config {
	my $self = shift;
	my $c = shift;

	# Check if filters are enabled, just return if not

	if ($self->{sys}->{loader}->{filter_enable} == 0)
	{
		$self->{sys}->{logmgr}->do_debug("AlteonMIB: $self->{device}->{hostname}: filter_config, filters_enabled==FALSE, skipping filters");
		return;
	}

	$self->{sys}->{logmgr}->do_debug("AlteonMIB: $self->{device}->{hostname}: filter_config, got config c=$c, searching array...");

	my $l = undef;
	my $f = undef;
	my $i = 0;
	my @truncate_list = ( "Configuration dump taken ", "usrpw ", "sopw ", "rcomm ", "wcomm ");

	# Check every config line
	foreach $l (@{$c})
	{
		# Check every truncate line
		foreach $f (@truncate_list)
		{
			if ($l =~ $f)
			{
				$self->{sys}->{logmgr}->do_debug("AlteonMIB: $self->{device}->{hostname}: filter_config() MATCH, i=$i, f=[$f]");

				my $ll = length($l);
				my $fl = length($f);
				my $fi = index($l, $f);

				$self->{sys}->{logmgr}->do_debug("AlteonMIB: $self->{device}->{hostname}: filter_config() truncate in progress, ll=$ll fl=$fl fi=$fi");

				${$c}[$i] = substr($l, 0, (0 - $ll) + ($fi + $fl))."[TRUNCATED]";
			}
		}

		$i++;
	}
}

1;