#!/usr/bin/perl -w

package NCS::CiscoMIB;
use strict;

use Net::SNMP;

# Define our Cisco SNMP object IDs
my %ccCopy = (
	method		=> ".1.3.6.1.4.1.9.9.96.1.1.1.1.2",
	source		=> ".1.3.6.1.4.1.9.9.96.1.1.1.1.3",
	dest		=> ".1.3.6.1.4.1.9.9.96.1.1.1.1.4",
	tftpsvr		=> ".1.3.6.1.4.1.9.9.96.1.1.1.1.5",
	filename	=> ".1.3.6.1.4.1.9.9.96.1.1.1.1.6",
	rowstatus	=> ".1.3.6.1.4.1.9.9.96.1.1.1.1.14",
	state		=> ".1.3.6.1.4.1.9.9.96.1.1.1.1.10",
	error		=> ".1.3.6.1.4.1.9.9.96.1.1.1.1.13",
);
my %ccCopyMethod = (
	tftp		=> "1",
);
my %ccCopyLocation = (
	tftp		=> "1",
	startup		=> "3",
	running		=> "4",
);
my %ccCopyState = (
	waiting		=> "1",
	running		=> "2",
	success		=> "3",
	failed		=> "4",
);
my %ccCopyError = (
		1	=> "Unknown copy fail code",
		2	=> "Bad file name",
		3	=> "Network timeout",
		4	=> "Not enough memory",
		5	=> "Source file does not exist",
);

sub new
{
	my $self = shift;
	my $sys = shift;
	my $device = shift;

	my $mib = {
		sys	=> $sys,
		device	=> $device,

		dstip	=> undef,
		dstfile	=> undef,

		error	=> undef,
	};
	bless($mib, $self);

	$sys->{logmgr}->do_debug("CiscoMIB: $device->{hostname}: successfully initialised");
	return $mib;
}

sub set_destination {
	my $self = shift;

	$self->{dstip} = shift;
	$self->{dstfile} = shift;

	$self->{sys}->{logmgr}->do_debug("CiscoMIB: $self->{device}->{hostname}: set_destination() dstip=$self->{dstip}, dstfile=$self->{dstfile}");
}

sub retrieve_config {
	my $self = shift;

	$self->{error} = undef;

	# Generate random number for MIB instances
	#
	srand(time | $$);
	my $r = int(rand(900)) + 10;

	#
	# Submit copy to tftp request
	#
	$self->{sys}->{logmgr}->do_debug("CiscoMIB: $self->{device}->{hostname}: doing SNMP Set...");
	$self->{device}->{snmp}->set(
			"$ccCopy{method}.$r",	INTEGER,	$ccCopyMethod{tftp},
			"$ccCopy{source}.$r",	INTEGER,	$ccCopyLocation{startup},
			"$ccCopy{dest}.$r",	INTEGER,	$ccCopyLocation{tftp},
			"$ccCopy{tftpsvr}.$r",	IPADDRESS,	$self->{dstip},
			"$ccCopy{filename}.$r",	OCTET_STRING,	$self->{dstfile},
			"$ccCopy{rowstatus}.$r",INTEGER, 	4
		);

	# Now we check whether we're successful

	if (defined($self->{device}->{snmp}->{error}) && $self->{device}->{snmp}->{error} ne "") {
		$self->{error} = "SNMP Set Failed: $self->{device}->{snmp}->{error}";
		return;
	}

	my $final_result = $ccCopyState{running};

	while (defined($final_result) && ($final_result eq "$ccCopyState{running}"
	  || $final_result eq "$ccCopyState{waiting}"))
	{
		$final_result = $self->{device}->{snmp}->get("$ccCopy{state}.$r");
	}

	$self->{sys}->{logmgr}->do_debug("CiscoMIB: $self->{device}->{hostname}: retrieve_config() final_result=".($final_result ? $final_result : "NULL"));

	if (!defined($final_result))
	{
		$self->{sys}->{logmgr}->do_debug("CiscoMIB: $self->{device}->{hostname}: *** ERROR *** !defined(final_result)");
		$self->{error} = "SNMP Failed: " . $self->{device}->{snmp}->{error};
	}
	elsif ($final_result ne $ccCopyState{success})
	{
		$self->{sys}->{logmgr}->do_debug("CiscoMIB: $self->{device}->{hostname}: *** ERROR *** final_result=".$final_result.", getting cause of error");
		$final_result = $self->{device}->{snmp}->get("$ccCopy{error}.$r");
		$self->{error} = "TFTP Transfer Failed: $ccCopyError{$final_result}";
	}

	# Clean up
	$self->{device}->{snmp}->set("$ccCopy{rowstatus}.$r", INTEGER, 6);
	$self->{sys}->{logmgr}->do_debug("CiscoMIB: $self->{device}->{hostname}: retrieve_config() finished, error=".($self->{error} ? $self->{error} : "NULL(Success)"));
}


sub filter_config {
	my $self = shift;
	my $c = shift;

	# Kludge: IOS seems to put a blank line at the top of a config file
	# Remove it here by checking if the len is zero
	#
	shift(@{$c}) if (($#{$c} > 0) && length(${$c}[0]) == 0);

	# Check if filters are enabled, just return if not
	if ($self->{sys}->{loader}->{filter_enable} == 0)
	{
		$self->{sys}->{logmgr}->do_debug("CiscoMIB: $self->{device}->{hostname}: filter_config: filter_enable=FALSE, skipping filters");
		return;
	}

	$self->{sys}->{logmgr}->do_debug("CiscoMIB: $self->{device}->{hostname}: filter_config, got config c=$c, searching array...");

	my $l = undef;
	my $f = undef;
	my $i = 0;
	my @truncate_list = ( "enable secret ", "password ", "ntp clock-period ");
	my @remove_one_list = ( "snmp-server community " );

	# Check every config line
	foreach $l (@{$c})
	{
		# Check every truncate line
		foreach $f (@truncate_list)
		{
			if ($l =~ $f)
			{
				$self->{sys}->{logmgr}->do_debug("CiscoMIB: $self->{device}->{hostname}: filter_config() MATCH(truncate), i=$i, f=[$f]");

				my $ll = length($l);
				my $fl = length($f);
				my $fi = index($l, $f);

				$self->{sys}->{logmgr}->do_debug("CiscoMIB: $self->{device}->{hostname}: filter_config() truncate in progress, ll=$ll fl=$fl fi=$fi");

				${$c}[$i] = substr($l, 0, (0 - $ll) + ($fi + $fl))."[TRUNCATED]";
			}
		}

		# Now check the "remove one" list

		$l = ${$c}[$i];
		foreach $f (@remove_one_list)
		{
			if ($l =~ $f)
			{
				$self->{sys}->{logmgr}->do_debug("CiscoMIB: $self->{device}->{hostname}: filter_config() MATCH(remove_one), i=$i, f=[$f]");

				my $ll = length($l);
				my $fl = length($f);
				my $fi = index($l, $f);
				my $fin = index($l, " ", ($fi + $fl));

				$self->{sys}->{logmgr}->do_debug("CiscoMIB: $self->{device}->{hostname}: filter_config() remove one in progress, ll=$ll fl=$fl fi=$fi fin=$fin");

				${$c}[$i] = substr($l, 0, ($fi + $fl))."[REMOVED]".substr($l, $fin);
			}
		}

		$i++;
	}
}

1;