#!/usr/bin/perl -w

package NCS::DB;
use strict;

use DBI;

sub new {
	my $self = shift;
	my $sys = shift;
	my $id = shift;

	my $db = {
		sys	=> $sys,
		id	=> $id,

		usernm	=> undef,
		passwd	=> undef,

		dbh	=> undef,

		error	=> undef
	};
	bless($db, $self);

	my $server = $sys->{ini}->val("db", "server")	|| undef;

	if (defined($server))
	{
		my $port = $sys->{ini}->val("db", "port")	|| 3306;
		my $dbname = $sys->{ini}->val("db", "database") || "ncs";

		# Load username/password for later
		#
		$db->{usernm} = $sys->{ini}->val("db", "username") || "";
		$db->{passwd} = $sys->{ini}->val("db", "password") || "";

		$db->{dsn} = "DBI:mysql:$dbname:$server:$port";
		$sys->{logmgr}->do_debug("DB: $id: ready [server=$server]");

		return $db;
	}

	$sys->{logmgr}->do_log("DB: $id: setup failed: db::server not specified by INI");
	return undef;
}

sub open {
	my $self = shift;

	if (defined($self->{dbi})) {
		$self->{sys}->{logmgr}->do_debug("DB: $self->{id}: open: context already initialised, ignoring request");
	} else {
		$self->{dbi} = DBI->connect($self->{dsn}, $self->{usernm}, $self->{passwd});

		if (!defined($self->{dbi}))
		{
			return $self->db_error("Failed to connect");
		}

		$self->{sys}->{logmgr}->do_debug("DB: $self->{id}: open: success, dsn=$self->{dsn}");
	}
	return 1;
}

sub close {
	my $self = shift;
	if (defined($self->{dbi})) {
		$self->{sys}->{logmgr}->do_debug("DB: $self->{id}: close: closing session");
		$self->{dbi}->disconnect;
	} else {
		$self->{sys}->{logmgr}->do_debug("DB: $self->{id}: close: context not initialised, ignoring request");
	}
	$self->{dbi} = undef;
}

sub test {
	my $self = shift;

	$self->{sys}->{logmgr}->do_debug("DB: $self->{id}: test in progress...");
	$self->close if $self->open;
	$self->{sys}->{logmgr}->do_debug("DB: $self->{id}: test completed!");
}

sub db_error {
	my $self = shift;
	my $errmsg = shift;

	if (!defined($self->{error}))
	{
		my $dbmsg = undef;

		if (DBI->err)
		{
			$dbmsg = DBI->errstr;
			$self->{error} = "$errmsg: $dbmsg";
		}
		else
		{
			$self->{error} = $errmsg;
		}
	}

	$self->{sys}->{logmgr}->do_debug("DB: $self->{id}: ERROR: $self->{error}");
	$self->{sys}->{logmgr}->do_log("DB: $self->{id}: ERROR: $self->{error}");

	return undef;
}

sub execute_query {
	my $self = shift;
	my $query = shift;
	my @params = @_;

	$self->open or return undef;

	my $handle = $self->{dbi}->prepare($query);
	if (!$handle)
	{
		return $self->db_error("Failed to prepare statement handle");
	}

	if (!$handle->execute(@params))
	{
		return $self->db_error("Failed to execute statement handle");
	}

	$self->{sys}->{logmgr}->do_debug("DB: $self->{id}: execute_query: handle OK, query=[$query]");

	return $handle;
}

sub get_record {
	my $self = shift;
	my $query = shift;
	my @params = @_;

	my $handle = $self->execute_query($query, @params);
	if ($handle)
	{
		my $record = $handle->fetchrow_hashref;
		if ($record) {
			return $record;
		}

		$self->{sys}->{logmgr}->do_debug("DB: $self->{id}: get_record: no records matched, query=[$query]");
	}

	return undef;
}

sub get_field {
	my $self = shift;
	my $query = shift;
	my @params = @_;

	my $handle = $self->execute_query($query, @params);
	if ($handle)
	{
		my $record = $handle->fetchrow_arrayref;
		if ($record) {
			return $$record[0];
		}

		$self->{sys}->{logmgr}->do_debug("DB: $self->{id}: get_field: no records matched, query=[$query]");
	}

	return undef;
}

sub add_record {
	my $self = shift;
	my $query = shift;
	my @params = @_;

	my $handle = $self->execute_query($query, @params);
	if ($handle)
	{
		return $self->{dbi}->{mysql_insertid};
	}

	return undef;
}

sub update_record {
	my $self = shift;
	my $query = shift;
	my @params = @_;

	my $handle = $self->execute_query($query, @params);
	if ($handle)
	{
		if ($handle->rows > 0) {
			return $handle->rows;
		}

		$self->{sys}->{logmgr}->do_debug("DB: $self->{id}: update_record failed, rows=".$handle->rows);
	}

	return undef;
}

1;
