#!/usr/bin/perl -w

package NCS::Device;
use strict;

use Config::IniFiles;
use File::Path;
use File::Basename;
use File::Compare;
use Net::SNMP;

use NCS::DB;
use NCS::Snmp;

use NCS::CiscoMIB;
use NCS::AlteonMIB;
use NCS::AlteonAppMIB;

# These are our device status codes which are generic
my %deviceState = (
	unchecked	=> 1,
	disabled	=> 2,
	initial		=> 3,
	changed		=> 4,
	unchanged	=> 5,
	failed		=> 6,
	syserr		=> 7,
	archived	=> 8
);
my %deviceType = (
	cisco		=> 1,
	alteonweb	=> 2,
	alteonapp	=> 3
);

# Platform independant SNMP objects
my %snmpObjects = (
	sysDescr	=> ".1.3.6.1.2.1.1.1.0"
);

sub new
{
	my $self = shift;
	my $sys = shift;
	my $device_id = shift;

	my $device = {
		sys => $sys,

		# ID and hostname of device
		device_id	=> $device_id,

		# DB handle for this device
		dbh		=> undef,

		# SNMP context for this device
		snmp		=> undef,

		# Device Settings (table fields)
		group_id	=> 0,
		revision_id	=> 0,
		alert_id	=> 0,
		hostname	=> undef,
		type_code	=> undef,
		ipaddress	=> undef,
		descr		=> undef,
		sys_descr	=> undef,
		state_code	=> undef,
		state_msg	=> undef,
		snmp_community	=> undef,
		snmp_v3		=> undef,
		snmp_username	=> undef,
		snmp_authproto	=> undef,
		snmp_authpass	=> undef,
		snmp_privproto	=> undef,
		snmp_privpass	=> undef,
		updated		=> undef,

		# MIB and SNMP Controllers
		mib		=> undef,

		# Location of config file on TFTP
		workfile	=> undef,

		# Stored configuration revision
		db_revision	=> undef,
		db_data		=> undef,

		# New configuration revision
		new_data	=> undef,
		new_revision	=> undef,
	};
	bless($device, $self);

	return $device;
}

sub set_state {
	my $self = shift;
	my $code = shift;
	my $msg = shift;

	$self->{state_code} = $code;
	$self->{state_msg} = $msg;

	$self->{sys}->{logmgr}->do_debug("Device: $self->{hostname}: set_state: code=$code, msg=".($msg ? $msg : "NULL"));

	# Don't bother trying to check for errors on this...

	$self->{dbh}->update_record("UPDATE `devices` SET `state_code` = ?, `state_msg` = ? WHERE `device_id` = ? LIMIT 1",
		$self->{state_code}, $self->{state_msg}, $self->{device_id});
	return undef;
}

sub handle_error {
	my $self = shift;

	if (!$self->{state_code})
	{
		$self->set_state($deviceState{syserr}, "A system error has occured");
	}

	$self->{sys}->{logmgr}->do_log("*** ERROR *** $self->{hostname}: $self->{state_msg}");
	$self->{dbh}->close;

	return 1; # Must be true
}

sub load_db {
	my $self = shift;

	$self->{sys}->{logmgr}->do_debug("Device: Loading record - device_id=$self->{device_id}");

	$self->{dbh} = new NCS::DB(\%{ $self->{sys} }, $self->{device_id});

	my $rec = $self->{dbh}->get_record("SELECT * FROM `devices` WHERE `device_id` = ?", $self->{device_id});
	if (!$rec)
	{
		$self->set_state($deviceState{failed}, $self->{dbh}->{error});
		$self->handle_error;
		return undef;
	}

	# For every column on the record, add to this context
	foreach my $key (keys(%{$rec})) {
		$self->{$key} = $$rec{$key};
	}

	return 1;
}


sub setup_mibctrl {
	my $self = shift;

	$self->{sys}->{logmgr}->do_debug("Device: $self->{hostname}: setup_mibctrl: loading controller, type_code=$self->{type_code}");

	if ($self->{type_code} == $deviceType{cisco}) {
		$self->{mib} = new NCS::CiscoMIB(\%{ $self->{sys} }, \%{ $self });
	}
	elsif ($self->{type_code} == $deviceType{alteonweb}) {
		$self->{mib} = new NCS::AlteonMIB(\%{ $self->{sys} }, \%{ $self });
	}
	elsif ($self->{type_code} == $deviceType{alteonapp}) {
		$self->{mib} = new NCS::AlteonAppMIB(\%{ $self->{sys} }, \%{ $self });
	}

	return $self->{mib};
}

sub setup_snmp {
	my $self = shift;

	$self->{snmp} = new NCS::Snmp(\%{ $self->{sys} });

	if ($self->{snmp_v3} == 1)
	{
		$self->{sys}->{logmgr}->do_debug("Device: $self->{hostname}: setup_snmp: snmp_v3=1, using SNMP::create_v3()");
		return $self->{snmp}->create_v3(
				$self->{ipaddress},
				$self->{snmp_username},
				$self->{snmp_authproto},
				$self->{snmp_authpass},
				$self->{snmp_privproto},
				$self->{snmp_privpass}
			);
	}

	$self->{sys}->{logmgr}->do_debug("Device: $self->{hostname}: setup_snmp: snmp_v3=0, using SNMP::create()");
	return $self->{snmp}->create($self->{ipaddress}, $self->{snmp_community});
}

sub setup_services {
	my $self = shift;

	# Setup any services we need...

	$self->{sys}->{logmgr}->do_debug("Device: $self->{hostname}: setup_services() called");

	if (!$self->setup_snmp) {
		return $self->set_state($deviceState{failed}, "Cannot setup SNMP Session: $self->{snmp}->{error}");
	}

	if (!$self->setup_mibctrl) {
		return $self->set_state($deviceState{syserr}, "Cannot setup MIB Controller: Invalid device type");
	}

	return 1;
}

sub get_sys_descr {
	my $self = shift;

	$self->{sys}->{logmgr}->do_debug("Device: $self->{hostname}: get_sys_descr() called");

	my $sys_descr = $self->{snmp}->get("$snmpObjects{sysDescr}");
	if (!defined($sys_descr))
	{
		$self->{sys}->{logmgr}->do_debug("Device: $self->{hostname}: get_sys_descr() snmp_get(sysDescr) returned NULL");
		return $self->set_state($deviceState{failed}, "Initial connection failed: $self->{snmp}->{error}");
	}

	my $updated = $self->{dbh}->update_record("UPDATE `devices` SET `sys_descr` = ? WHERE `device_id` = ? LIMIT 1;",
		$sys_descr, $self->{device_id});
	if (!$updated || $updated == 0)
	{
		$self->{sys}->{logmgr}->do_debug("Device: $self->{hostname}: get_sys_descr() update(devices.sys_descr) failed!");
		return $self->set_state($deviceState{syserr}, "UPDATE Failed: devices.sys_descr");
	}

	$self->{sys}->{logmgr}->do_debug("Device: $self->{hostname}: get_sys_descr() updated=$updated");
	$self->{sys_descr} = $sys_descr;

	return 1;
}

sub device_process {
	my $self = shift;

	$self->{sys}->{logmgr}->do_debug("Device: $self->{hostname}: processing, state_code=$self->{state_code}");

	if ($self->{state_code} eq $deviceState{disabled} || $self->{state_code} eq $deviceState{archived})
	{
		$self->{sys}->{logmgr}->do_debug("Device: $self->{hostname}: device state is DISABLED or ARCHIVED");
		goto ABORT;
	}

	$self->{sys}->{logmgr}->do_log("Device: $self->{hostname} is being processed");

	# Setup MIB controller and SNMP context
	$self->setup_services or ($self->handle_error and goto ABORT);

	# Get system information
	$self->get_sys_descr or ($self->handle_error and goto ABORT);

	# Now begin to do some config work...
	$self->load_new_config or ($self->handle_error and goto ABORT);
	$self->load_db_config or ($self->handle_error and goto ABORT);

	$self->compare_config;

	if ($self->{state_code} == $deviceState{changed} || $self->{state_code} == $deviceState{initial})
	{
		$self->{sys}->{logmgr}->do_log("$self->{hostname}: saving new configuration to database...");
		$self->save_new_config or ($self->handle_error and goto ABORT);
	}

	$self->{sys}->{logmgr}->do_log("$self->{hostname}: successfully processed.");

	ABORT:
	$self->{dbh}->close;
	exit;
}

sub load_db_config {
	my $self = shift;

	if (defined($self->{revision_id}))
	{
		my $rec = $self->{dbh}->get_record("SELECT `revision`, `data` FROM `revisions` WHERE `revision_id` = ? LIMIT 1",
			$self->{revision_id});
		if (!$rec) {
			return $self->set_state($deviceState{syserr}, "Cannot load current revision from database");
		}

		$self->{db_revision} = $$rec{revision};
		$self->{db_data} = $$rec{data};
		$self->{sys}->{logmgr}->do_debug("Device: $self->{hostname}: load_db_config() revision_id=$self->{revision_id} ".
			"revision=$self->{db_revision} loaded");

		$self->{new_revision} = ($self->{db_revision} + 1);
	}
	else
	{
		$self->{sys}->{logmgr}->do_debug("Device: $self->{hostname}: load_db_config() revision_id=NULL, this is initial configuration!");
		$self->{new_revision} = 1;
	}
	return 1;
}

sub load_new_config {
	my $self = shift;

	$self->{workfile} = $self->{sys}->{loader}->{tftp_path}."/".$self->{device_id}.".tmp";
	$self->{mib}->set_destination($self->{sys}->{loader}->{tftp_ipaddress}, $self->{device_id}.".tmp");

	my $transCount = 0;

	while ($transCount < 2)
	{
		unlink($self->{workfile}) if (-f $self->{workfile});
		$self->{sys}->{logmgr}->do_debug("Device: $self->{hostname}: transfer_config attempting to mib::retrieve_config(), transCount=$transCount");

		$self->{mib}->retrieve_config;

		if (!defined($self->{mib}->{error})) {
			# No error, we're done here so break out.
			last;
		}

		$transCount++;
	}

	$self->{snmp}->close;

	if (defined($self->{mib}->{error}))
	{
		$self->{sys}->{logmgr}->do_debug("Device: $self->{hostname}: transCount limit reached, state is FAILED");
		unlink($self->{workfile}); # Remove any dead work file
		return $self->set_state($deviceState{failed}, "Failed to transfer configuration: $self->{mib}->{error}")
	}

	# Process the downloaded config
	my @config = $self->{sys}->{util}->read_file($self->{workfile});
	unlink($self->{workfile});

	# Filter via the MIB controller
	$self->{mib}->filter_config(\@config);

	# Join into a data buffer
	$self->{new_data} = join("\n", @config);

	return 1;
}

sub compare_config {
	my $self = shift;

	if (!defined($self->{db_data}))
	{
		$self->{sys}->{logmgr}->do_debug("Device: $self->{hostname}: db_data=NULL, state is INITIAL");
		return $self->set_state($deviceState{initial});
	}

	if ($self->{db_data} ne $self->{new_data})
	{
		$self->{sys}->{logmgr}->do_debug("Device: $self->{hostname}: db_data NE new_data, state is CHANGED");
		return $self->set_state($deviceState{changed});
	}

	$self->{sys}->{logmgr}->do_debug("Device: $self->{hostname}: db_data EQ new_data, state is UNCHANGED");
	return $self->set_state($deviceState{unchanged});
}

sub save_new_config {
	my $self = shift;

	if (!defined($self->{new_revision}) || !defined($self->{new_data})) {
		return $self->set_state($deviceState{syserr}, "Cannot save new configuration: invalid data");
	}

	# Insert the revision and get the new record ID

	my $new_id = $self->{dbh}->add_record("INSERT INTO `revisions` (`device_id`, `revision`, `data`) VALUES (?, ?, ?);",
			$self->{device_id}, $self->{new_revision}, $self->{new_data});

	if (!$new_id)
	{
		$self->{sys}->{logmgr}->do_debug("Device: $self->{hostname}: save_new_config() insert(revisions) FAIL, abort");
		return $self->set_state($deviceState{syserr}, "Revision INSERT Failed");
	}

	# Now we update the devices.revision_id field pointing to the new revision record...

	my $updated = $self->{dbh}->update_record("UPDATE `devices` SET `revision_id` = ? WHERE `device_id` = ? LIMIT 1;",
		$new_id, $self->{device_id});
	if (!$updated || $updated == 0)
	{
		$self->{sys}->{logmgr}->do_debug("Device: $self->{hostname}: save_new_config() update(devices) FAIL, updated=".
			($updated ? $updated : "NULL"));
		return $self->set_state($deviceState{syserr}, "Device UPDATE Failed");
	}

	# We successfully stored a new configuration
	$self->set_state($deviceState{changed});
	return 1;
}

1;