#!/usr/bin/perl -w

package NCS::Loader;
use strict;

use Getopt::Long;
use NCS::Device;
use NCS::Alert;

sub new {
	my $self = shift;
	my $sys = shift;

	my $loader = {
		sys		=> $sys,

		devices		=> undef,
		alerts		=> undef,

		# ncs.options.*
		#
		version		=> undef,
		url		=> undef,
		max_threads	=> undef,
		tftp_svc_name	=> undef,
		tftp_path	=> undef,
		tftp_ipaddress	=> undef,
		log_path	=> undef,
		log_enable	=> undef,
		log_debug	=> undef,
		log_max_history	=> undef,
		filter_enable	=> undef,
		alert_smtp_server => undef,
		alert_source	=> undef	 	 	
	};

	bless ($loader, $self);
	return $loader;
}

sub parse_options {
	my %opt;
	my @opts = qw(
			version
			testdb
		);

	foreach (@opts) {
		my $option = $_;

		$option =~ s/[!+]//;
		$option =~ s/[=:]\w+//;

		$opt{$option} = '';
	}

	GetOptions(\%opt, @opts);

	return \%opt;
}

sub connect_database {
	my $self = shift;

	$self->{sys}->{logmgr}->do_debug("Loader: connect_database: connecting...");
	if (!$self->{sys}->{db}->open)
	{
		$self->{sys}->{logmgr}->do_log("*** FATAL: CANNOT CONNECT TO DATABASE, ABORTING! ***");
		return undef;
	}

	return 1;
}

sub load_options_db {
	my $self = shift;

	my $h = $self->{sys}->{db}->get_record("SELECT * FROM `options`");
	if (!$h)
	{
		$self->{sys}->{logmgr}->do_log("*** FATAL: CANNOT LOAD OPTIONS, ABORTING! ***");
		return undef;
	}

	#
	# Put all fields into the loader...
	#
	foreach my $key (keys(%{$h})) {
		$self->{$key} = $$h{$key};
	}

	#
	# Now check we have everything we need...
	#
	if (!defined($self->{log_path})) {
		$self->{sys}->{logmgr}->do_log("*** FATAL: Log Path is currently undefined or does not exist ***");
		return undef;
	}
	if (!defined($self->{max_threads}) || $self->{max_threads} < 0) {
		$self->{sys}->{logmgr}->do_log("*** FATAL: Maximum Threads is currently undefined or negative; must be positive integer ***");
		return undef;
	}
	if (!defined($self->{tftp_path}) || !(-d $self->{tftp_path})) {
		$self->{sys}->{logmgr}->do_log("*** FATAL: TFTP Path is currently undefined or does not exist ***");
		return undef;
	}
	if (!defined($self->{tftp_ipaddress})) {
		$self->{sys}->{logmgr}->do_log("*** FATAL: TFTP IP Address is currently undefined ***");
		return undef;
	}

	return 1;
}

sub load_device_db {
	my $self = shift;

	#
	# Load all device record ID's
	#

	$self->{sys}->{logmgr}->do_debug("Loader: reading device table...");
	my $h = $self->{sys}->{db}->execute_query("SELECT `device_id` FROM `devices` ORDER BY `device_id` ASC");
	if (!$h)
	{
		$self->{sys}->{logmgr}->do_log("*** FATAL: CANNOT READ DEVICES, ABORTING! ***");
		return undef;
	}

	#
	# Load the blank CSDevice objects
	#

	$self->{sys}->{logmgr}->do_debug("Loader: loading device table...");
	my @devices = ();

	while (defined(my $row = $h->fetchrow_arrayref))
	{
		my $device = new NCS::Device(\%{ $self->{sys} }, $$row[0]);
		push(@devices, $device);
	}

	$self->{devices} = \@devices;
	$self->{sys}->{logmgr}->do_log("Loaded ".scalar(@devices)." device records");

	return 1;
}

sub load_alerts_db {
	my $self = shift;

	if (!defined($self->{alert_smtp_server}) || !defined($self->{alert_source}))
	{
		$self->{sys}->{logmgr}->do_log("Alerts are disabled - missing SMTP Server or Alert Source.");
		return 1;
	}

	$self->{sys}->{logmgr}->do_debug("Loader: reading alert groups table..");
	my $h = $self->{sys}->{db}->execute_query("SELECT `alert_id`,`name`,`enabled` FROM `alerts` ORDER BY `alert_id` ASC");
	if (!$h)
	{
		$self->{sys}->{logmgr}->do_log("*** FATAL: CANNOT READ ALERT GROUPS, ABORTING! ***");
		return undef;
	}

	$self->{sys}->{logmgr}->do_debug("Loader: loading alert groups table...");
	my @alerts = ();

	while (defined(my $row = $h->fetchrow_arrayref))
	{
		my $alert = new NCS::Alert(\%{ $self->{sys} }, $$row[0], $$row[1], $$row[2]);
		if ($alert->load_targets)
		{
			push(@alerts, $alert);
		}
	}

	$self->{alerts} = \@alerts;
	$self->{sys}->{logmgr}->do_log("Loaded ".scalar(@alerts)." alert records");

	return 1;
}


##### TFTP Server Control

use constant SVC_NOT_CONTROLLED	=> -2; # NCS does not control this service.
use constant SVC_APP_ERROR	=> -1; # Custom Error

# API error codes
use constant SVC_STOPPED	=> 1;
use constant SVC_START_PEND	=> 2;
use constant SVC_STOP_PEND	=> 3;
use constant SVC_STARTED	=> 4;
use constant SVC_CONT_PEND	=> 5;
use constant SVC_PAUSE_PEND	=> 6;
use constant SVC_PAUSED		=> 7;
use constant SVC_ERROR		=> 8;

sub get_tftp_status {
	my $self = shift;
	my %status;
	my $result;

	return SVC_NOT_CONTROLLED if (!defined($self->{tftp_svc_name}) || $self->{tftp_svc_name} eq "");

	$result = Win32::Service::GetStatus("", $self->{tftp_svc_name}, \%status);
	if (!$result)
	{
		sleep(1);

		# Try again
		$result = Win32::Service::GetStatus("", $self->{tftp_svc_name}, \%status);
	}

	$self->{sys}->{logmgr}->do_debug("Loader: get_tftp_status: res=".($result ? $result : "NULL"));

	return SVC_APP_ERROR if (!$result);
	return $status{CurrentState};
}

sub svc_blocking {
	my $self = shift;
	my $status = shift;

	if ($status eq SVC_START_PEND || $status eq SVC_STOP_PEND
	  || $status eq SVC_CONT_PEND || $status eq SVC_PAUSE_PEND || $status eq SVC_PAUSED) {
		$self->{sys}->{logmgr}->do_debug("Loader: svc_blocking: blocking status=$status");
		return 1;
	}

	$self->{sys}->{logmgr}->do_debug("Loader: svc_blocking: status=$status, ignoring API failure");
	return undef;
}

sub start_tftp_svc {
	my $self = shift;
	my $status = $self->get_tftp_status;

	if ($status == SVC_NOT_CONTROLLED) {
		return 1;
	}

	if ($status == SVC_STARTED) {
		$self->{sys}->{logmgr}->do_debug("Loader: start_tftp_svc: TFTP Server already running [$self->{tftp_svc_name}]");
		return 1;
	}

	if (!$self->svc_blocking($status))
	{
		$self->{sys}->{logmgr}->do_debug("Loader: starting TFTP Server [$self->{tftp_svc_name}]...");

		my $result = Win32::Service::StartService("", $self->{tftp_svc_name});
		my $sleep = 10;

		while ($sleep-- > 0)
		{
			sleep(1);

			if ($self->get_tftp_status == SVC_STARTED)
			{
				$self->{sys}->{logmgr}->do_log("TFTP Server successfully started [$self->{tftp_svc_name}]") and return 1;
			}
		}

		$self->{sys}->{logmgr}->do_log("TFTP Server failed to start [$self->{tftp_svc_name}]");
	}
	else
	{
		$self->{sys}->{logmgr}->do_log("*** APPLICATION ERROR *** start_tftp_svc() with blocking API call, status=$status, svc_name=$self->{tftp_svc_name}");
		$self->{sys}->{logmgr}->do_log("FATAL ERROR: A previous Win32::Service operation has not completed - service cannot be started.");
	}

	return undef;
}

sub stop_tftp_svc {
	my $self = shift;
	my $status = $self->get_tftp_status;

	if ($status == SVC_NOT_CONTROLLED) {
		return 1;
	}

	if ($status == SVC_STOPPED) {
		$self->{sys}->{logmgr}->do_debug("Loader: stop_tftp_svc: TFTP Server not running [$self->{tftp_svc_name}]");
		return 1;
	}

	if (!$self->svc_blocking($status))
	{
		$self->{sys}->{logmgr}->do_debug("Loader: stopping TFTP Server [$self->{tftp_svc_name}]...");

		my $result = Win32::Service::StopService("", $self->{tftp_svc_name});
		my $sleep = 10;

		while ($sleep-- > 0)
		{
			sleep(1);

			if ($self->get_tftp_status == SVC_STOPPED)
			{
				$self->{sys}->{logmgr}->do_log("TFTP Server successfully stopped [$self->{tftp_svc_name}]") and return 1;
			}
		}

		$self->{sys}->{logmgr}->do_log("TFTP Server failed to stop [$self->{tftp_svc_name}]");
	}
	else
	{
		$self->{sys}->{logmgr}->do_log("*** APPLICATION ERROR *** stop_tftp_svc() with blocking API call, status=$status, svc_name=$self->{tftp_svc_name}");
		$self->{sys}->{logmgr}->do_log("FATAL ERROR: A previous Win32::Service operation has not completed - service cannot be stopped.");
	}

	return undef;
}

1;