#!/usr/bin/perl -w

package NCS::LogMgr;
use strict;

use File::Basename;
use File::stat;
use File::Path;

sub new
{
	my $self = shift;
	my $sys = shift;

	my($sec, $min, $hr, $mday, $mon, $yr) = localtime();

	my $date = sprintf("%04d-%02d-%02d", $yr+1900, $mon, $mday);
	my $time = sprintf("%02d-%02d-%02d", $hr, $min, $sec);

	my $logmgr = {
		sys		=> $sys,
		date		=> $date,
		time		=> $time,

		path		=> undef,
		logfile		=> undef,
		debugfile	=> undef,

		enabled		=> 0,
	};
	bless($logmgr, $self);

	return $logmgr;
}

sub setup_logdir {
	my $self = shift;

	return unless ($self->{enabled} == 0); # We can only setup once.

	# Let's see what paths we're dealing with.
	#
	if (defined($self->{sys}->{loader}->{log_path}) && $self->{sys}->{loader}->{log_path} ne "")
	{
		$self->{path} = $self->{sys}->{loader}->{log_path}."/".$self->{date}.".logdir";
		$self->{logfile} = $self->{path}."/".$self->{time}.".log";
		$self->{debugfile} = $self->{path}."/".$self->{time}.".debug";

		# Create root log path
		mkpath($self->{sys}->{loader}->{log_path}) unless (-e $self->{sys}->{loader}->{log_path});

		# Create log directory for today
		mkpath($self->{path}) unless (-e $self->{path});

		# Enable this log manager
		$self->{enabled} = 1;
	}
}

sub rotate_logdirs {
	my $self = shift;

	return unless ($self->{enabled} == 1); # Ensure we are set up...
	return unless ($self->{sys}->{loader}->{log_max_history} > 0); # Because we're lazy...

	my $location = $self->{sys}->{loader}->{log_path}; # Kludge: Perl 5.8 doesn't like hash references in glob
	my @folders = <$location/*>;
	my %logdirs = ();

	$self->do_debug("LogMgr: rotating log directories in [$location] NOW...");

	foreach(@folders)
	{ 
		next unless -d;
		$self->do_debug("LogMgr: rotate_logdirs: parsing directory: $_");

		my $file = basename($_);
		my $dotloc = index($file, ".");
		next unless ($dotloc > 1  && substr($file, $dotloc+1) =~ m/logdir/);

		$self->do_debug("LogMgr: rotate_logdirs: adding to logdir hash: file=$file [$_]");

		#
		# Get the modified time of this folder and use it as an
		# index into the %logdirs hash array.
		#
		my $l = stat($_);
		$logdirs{$l->mtime} = $_;
	}

	# Sort the filedirs hash array in decreasing order which will put newer files at the
	# top. Then walk the list and start deleting folders after log_max_history elements.
	#

	my $l = 0;
	foreach my $i (reverse(sort(keys(%logdirs))))
	{
		$l++;
		$self->do_debug("LogMgr: rotate_logdirs: parsing logdirs i(mt)=$i l=$l max=$self->{sys}->{loader}->{log_max_history} dir=$logdirs{$i}");

		if ($l > $self->{sys}->{loader}->{log_max_history})
		{
			$self->do_log("Deleting log folder $logdirs{$i}: exceeds maximum history");
			rmtree($logdirs{$i}, 0, 1);
		}
	}
}

sub append_log {
	my $self = shift;
	my $file = shift;
	my $line = shift;

	my $log = sprintf("[%s] %s", $self->{sys}->{util}->nowstr, $line);

	if ($self->{enabled} == 0 || (defined($self->{sys}->{loader}->{log_enable}) && $self->{sys}->{loader}->{log_enable} == 0)) {
		print("$log\n");
	} else {
		$self->{sys}->{util}->append_file($file, $log);
	}
}

sub do_log {
	my $self = shift;
	my $line = shift;

	$self->append_log($self->{logfile}, $line);
}

sub do_debug {
	my $self = shift;
	my $line = shift;

	if (defined($self->{sys}->{loader}->{log_debug}))
	{
		if ($self->{sys}->{loader}->{log_debug} == 1)
		{
			$self->append_log($self->{debugfile}, "[DEBUG] ".$line);
		}
	}
}

1;