#!/usr/bin/perl -w

package NCS::Snmp;
use strict;

use Net::SNMP;
use Net::SNMP::Security::USM v3.0.0;

# These map to the auth/priv modes in the GUI
my %V3AuthModes = (
			0 => "none",
			1 => "md5",
			2 => "sha"
		);
my %V3PrivModes = (
			0 => "none",
			1 => "des",
			2 => "aes",
			3 => "3des"
		);

sub new {
	my $self = shift;
	my $sys = shift;

	my $snmp = {
		sys	=> $sys,
		session	=> undef,
		error	=> "",
	};
	bless($snmp, $self);

	return $snmp;
}

sub create_finish {
	my $self = shift;

	if (!defined($self->{session}))
	{
		if ($self->{error} eq "") {
			$self->{error} = "Invalid session";
		}

		return undef;
	}

	return 1;
}

sub create {
	my $self = shift;
	my $ip = shift;
	my $comm = shift;

	my %opts = ();

	$opts{version} = 1;
	$opts{hostname} = $ip;
	$opts{community} = $comm;

	$opts{retries} = $self->{sys}->{ini}->val("snmp", "retries")	|| "1";
	$opts{timeout} = $self->{sys}->{ini}->val("snmp", "timeout")	|| "5.0";
	$opts{maxmsgsize} = $self->{sys}->{ini}->val("snmp", "mtu")	|| "1500";
	$opts{debug} = $self->{sys}->{ini}->val("snmp", "debug")	|| "0";

	($self->{session}, $self->{error}) = Net::SNMP->session(%opts);
	return $self->create_finish;
}

sub create_v3 {
	my $self = shift;
	my $ip = shift;
	my $username = shift;
	my $authproto = shift;
	my $authpass = shift;
	my $privproto = shift;
	my $privpass = shift;

	my %opts = ();

	$opts{version} = 3;
	$opts{hostname} = $ip;

	if ($username) {
		$opts{username} = $username;
		$self->{sys}->{logmgr}->do_debug("SNMP: create_v3() hostname=$ip username is defined");
	}

	if ($authproto && ($authproto ne "none"))
	{
		$opts{authprotocol} = $V3AuthModes{$authproto};
		$self->{sys}->{logmgr}->do_debug("SNMP: create_v3() hostname=$ip with authentication, protocol=$authproto");

		if ($authpass) {
			$opts{authpassword} = $authpass;
			$self->{sys}->{logmgr}->do_debug("SNMP: create_v3() hostname=$ip with authentication, password is defined");
		}
	}

	if ($privproto && ($privproto ne "none"))
	{
		$opts{privprotocol} = $V3AuthModes{$privproto};
		$self->{sys}->{logmgr}->do_debug("SNMP: create_v3() hostname=$ip with privacy, protocol=$privproto");

		if ($privpass) {
			$opts{privpassword} = $privpass;	
			$self->{sys}->{logmgr}->do_debug("SNMP: create_v3() hostname=$ip with privacy, password is defined");
		}
	}

	$opts{retries} = $self->{sys}->{ini}->val("snmp", "retries")	|| "1";
	$opts{timeout} = $self->{sys}->{ini}->val("snmp", "timeout")	|| "5.0";
	$opts{maxmsgsize} = $self->{sys}->{ini}->val("snmp", "mtu")	|| "1500";
	$opts{debug} = $self->{sys}->{ini}->val("snmp", "debug")	|| "0";

	($self->{session}, $self->{error}) = Net::SNMP->session(%opts);
	return $self->create_finish;
}

sub close {
	my $self = shift;

	if (defined($self->{session})) {
		$self->{session}->close;
		$self->{session} = undef;
	}

	return 1;
}

sub get {
	my $self = shift;
	my $oid = shift;


	if (defined($self->{session})) {
		my $ret = $self->{session}->get_request("$oid");
		$self->{error} = $self->{session}->error;

		return $ret->{$oid} if ($self->{error} eq "");
	}
	else {
		$self->{sys}->{logmgr}->do_debug("SNMP: get() with uninitialised session, oid=$oid");
	}

	return undef;
}

sub set {
	my $self = shift;

	if (defined($self->{session})) {
		my @oids = @_;
		my $ret = $self->{session}->set_request(@oids);

		return $ret if (defined($ret));

		$self->{error} = $self->{session}->error;
	}
	else {
		$self->{sys}->{logmgr}->do_debug("SNMP: set() with uninitialised session");
	}

	return undef;
}

1;
